@@ -1,16 +1,17 @@
 <?php
 
 /* vnstat-json.php -- example php for vnStat json output */
-/* copyright (c) 2015 Teemu Toivola <tst at iki dot fi> */
+/* copyright (c) 2015-2021 Teemu Toivola <tst at iki dot fi> */
 /* released under the GNU General Public License */
 
 
-/* list of available interfaces, edit as necessary */
-$interfaces = array("eth0", "eth1", "ethX");
-
 /* location of vnstat binary */
 $vnstat_cmd = "/usr/bin/vnstat";
 
+/* individually accessible interfaces with ?interface=N */
+/* for static list, uncomment first line below, update the list and comment out second line */
+//$interfaces = array("eth0", "eth1");
+$interfaces = explode("\n", trim(shell_exec("$vnstat_cmd --dbiflist 1")));
 
 /* no editing should be needed below this line */
 
