@@ -44,6 +44,7 @@ int loadcfg(const char *cfgfile, const ConfigType type)
 		 {"ListMonths", 0, &cfg.listmonths, 0, 0},
 		 {"ListYears", 0, &cfg.listyears, 0, 0},
 		 {"ListTop", 0, &cfg.listtop, 0, 0},
+		 {"InterfaceMatchMethod", 0, &cfg.ifacematchmethod, 0, 0},
 		 {"5MinuteHours", 0, &cfg.fiveminutehours, 0, 0},
 		 {"HourlyDays", 0, &cfg.hourlydays, 0, 0},
 		 {"DailyDays", 0, &cfg.dailydays, 0, 0},
@@ -57,6 +58,7 @@ int loadcfg(const char *cfgfile, const ConfigType type)
 		 {"PollInterval", 0, &cfg.pollinterval, 0, 0},
 		 {"SaveInterval", 0, &cfg.saveinterval, 0, 0},
 		 {"OfflineSaveInterval", 0, &cfg.offsaveinterval, 0, 0},
+		 {"RescanDatabaseOnSave", 0, &cfg.rescanonsave, 0, 0},
 		 {"AlwaysAddNewInterfaces", 0, &cfg.alwaysadd, 0, 0},
 		 {"BandwidthDetectionInterval", 0, &cfg.bwdetectioninterval, 0, 0},
 		 {"SaveOnStatusChange", 0, &cfg.savestatus, 0, 0},
@@ -78,6 +80,7 @@ int loadcfg(const char *cfgfile, const ConfigType type)
 		 {"ImageScale", 0, &cfg.imagescale, 0, 0},
 		 {"5MinuteGraphResultCount", 0, &cfg.fivegresultcount, 0, 0},
 		 {"5MinuteGraphHeight", 0, &cfg.fivegheight, 0, 0},
+		 {"HourlyGraphMode", 0, &cfg.hourlygmode, 0, 0},
 		 {"SummaryGraph", 0, &cfg.summarygraph, 0, 0},
 		 {"EstimateStyle", 0, &cfg.estimatestyle, 0, 0},
 		 {"BarColumnShowsRate", 0, &cfg.barshowsrate, 0, 0},
@@ -114,7 +117,7 @@ int loadcfg(const char *cfgfile, const ConfigType type)
 		}
 
 		linelen = (unsigned int)strlen(cfgline);
-		if (linelen <= 2 || cfgline[0] == '#') {
+		if (linelen <= 2 || cfgline[0] == '#' || cfgline[0] == ';') {
 			continue;
 		}
 
@@ -215,6 +218,7 @@ void validatecfg(const ConfigType type)
 		validateint("ImageScale", &cfg.imagescale, IMAGESCALE, 50, 500);
 		validateint("5MinuteGraphResultCount", &cfg.fivegresultcount, FIVEGRESULTCOUNT, FIVEGMINRESULTCOUNT, 2000);
 		validateint("5MinuteGraphHeight", &cfg.fivegheight, FIVEGHEIGHT, FIVEGMINHEIGHT, 2000);
+		validateint("HourlyGraphMode", &cfg.hourlygmode, HOURLYGMODE, 0, 1);
 		validatebool("SummaryGraph", &cfg.summarygraph, SUMMARYGRAPH);
 		validateint("EstimateStyle", &cfg.estimatestyle, ESTIMATESTYLE, 0, 2);
 		validatebool("BarColumnShowsRate", &cfg.barshowsrate, BARSHOWSRATE);
@@ -228,6 +232,7 @@ void validatecfg(const ConfigType type)
 	validateint("ListMonths", &cfg.listmonths, LISTMONTHS, 0, 0);
 	validateint("ListYears", &cfg.listyears, LISTYEARS, 0, 0);
 	validateint("ListTop", &cfg.listtop, LISTTOP, 0, 0);
+	validateint("InterfaceMatchMethod", &cfg.ifacematchmethod, IFACEMATCHMETHOD, 0, 3);
 	validateint("5MinuteHours", &cfg.fiveminutehours, FIVEMINUTEHOURS, -1, -1);
 	validateint("HourlyDays", &cfg.hourlydays, HOURLYDAYS, -1, -1);
 	validateint("DailyDays", &cfg.dailydays, DAILYDAYS, -1, -1);
@@ -358,6 +363,7 @@ void defaultcfg(void)
 	cfg.summaryrate = SUMMARYRATE;
 	cfg.trafficlessentries = TRAFFICLESSENTRIES;
 	cfg.utflocale = UTFLOCALE;
+	cfg.ifacematchmethod = IFACEMATCHMETHOD;
 
 	cfg.listfivemins = LISTFIVEMINS;
 	cfg.listhours = LISTHOURS;
@@ -413,6 +419,7 @@ void defaultcfg(void)
 	cfg.imagescale = IMAGESCALE;
 	cfg.fivegresultcount = FIVEGRESULTCOUNT;
 	cfg.fivegheight = FIVEGHEIGHT;
+	cfg.hourlygmode = HOURLYGMODE;
 	cfg.summarygraph = SUMMARYGRAPH;
 	cfg.estimatestyle = ESTIMATESTYLE;
 	cfg.barshowsrate = BARSHOWSRATE;
@@ -429,6 +436,7 @@ void defaultcfg(void)
 	strncpy_nt(cfg.ctx, CTX, 8);
 	strncpy_nt(cfg.ctxd, CTXD, 8);
 
+	cfg.timestampprints = 0;
 	cfg.experimental = 0;
 }
 
@@ -540,17 +548,19 @@ int setcfgvalue(const struct cfgsetting *cset, const char *value, const char *cf
 
 void configlocale(void)
 {
+	const char *env = getenv("LC_ALL");
+
 	if (cfg.locale[0] != '-' && strlen(cfg.locale) > 0) {
 		setlocale(LC_ALL, cfg.locale);
 	} else {
-		if (getenv("LC_ALL")) {
-			setlocale(LC_ALL, getenv("LC_ALL"));
+		if (env) {
+			setlocale(LC_ALL, env);
 		} else {
 			setlocale(LC_ALL, "");
 		}
 	}
-	if (getenv("LC_ALL")) {
-		if (strstr(getenv("LC_ALL"), "UTF") != NULL) {
+	if (env) {
+		if (strstr(env, "UTF") != NULL) {
 			cfg.utflocale = 1;
 		} else {
 			cfg.utflocale = 0;
