@@ -154,6 +154,11 @@ and most can be changed later from the config file.
 /* 7 = hours, 8 = xml, 9 = one line, 10 = json */
 #define DEFQMODE 0
 
+/* interface match method */
+/* 0 = interface name exact case sensitive, 1 = 0 + case sensitive exact alias */
+/* 2 = 1 + case insensitive exact alias, 3 = 2 + case insensitive beginning of alias */
+#define IFACEMATCHMETHOD 3
+
 /* how much the boot time can variate between updates (seconds) */
 #define BVAR 15
 
@@ -272,6 +277,9 @@ and most can be changed later from the config file.
 #define FIVEGMINRESULTCOUNT 288
 #define FIVEGMINHEIGHT 150
 
+/* hourly graph mode (0 = 24 hour sliding window, 1 = begins from midnight) */
+#define HOURLYGMODE 0
+
 /* graph used in extended summary, 0 = hours, 1 = 5 minutes*/
 #define SUMMARYGRAPH 0
 
@@ -303,18 +311,18 @@ typedef struct {
 	char rxchar[2], txchar[2], rxhourchar[2], txhourchar[2];
 	char cbg[8], cedge[8], cheader[8], cheadertitle[8], cheaderdate[8], ctext[8];
 	char cline[8], clinel[8], cvnstat[8], crx[8], crxd[8], ctx[8], ctxd[8];
-	int32_t unitmode, rateunitmode, rateunit, bvar, qmode, sampletime, hourlyrate, summaryrate;
+	int32_t unitmode, rateunitmode, rateunit, bvar, qmode, ifacematchmethod, sampletime, hourlyrate, summaryrate;
 	int32_t monthrotate, monthrotateyears, maxbw, spacecheck, trafficlessentries, transbg, ostyle;
 	int32_t defaultdecimals, hourlydecimals, hourlystyle, is64bit, waldb, dbsynchronous, useutc, imagescale;
 	int32_t largefonts, linespaceadjust, estimatebarvisible, estimatestyle, barshowsrate, fivegresultcount;
-	int32_t fivegheight, summarygraph, alwaysadd;
+	int32_t fivegheight, summarygraph, hourlygmode, alwaysadd;
 	char cfgfile[512], logfile[512], pidfile[512];
 	char daemonuser[33], daemongroup[33];
 	int32_t timesyncwait, updateinterval, pollinterval, saveinterval, offsaveinterval, rescanonsave, savestatus;
 	int32_t uselogging, createdirs, updatefileowner, bwdetection, bwdetectioninterval, utflocale;
 	int32_t fiveminutehours, hourlydays, dailydays, monthlymonths, yearlyyears, topdayentries;
 	int32_t listfivemins, listhours, listdays, listmonths, listyears, listtop, listjsonxml;
-	int32_t experimental;
+	int32_t timestampprints, experimental;
 } CFG;
 
 /* internal interface information structure */
