@@ -252,7 +252,6 @@ void initdstate(DSTATE *s)
 	s->sync = 0;
 	s->forcesave = 0;
 	s->noadd = 0;
-	s->alwaysadd = 0;
 	s->initdb = 0;
 	s->iflisthash = 0;
 	s->cfgfile[0] = '\0';
@@ -273,7 +272,7 @@ void preparedatabase(DSTATE *s)
 {
 	s->dbifcount = db_getinterfacecount();
 
-	if (s->dbifcount > 0 && !s->alwaysadd) {
+	if (s->dbifcount > 0 && !cfg.alwaysadd) {
 		s->dbifcount = 0;
 		return;
 	}
@@ -293,7 +292,7 @@ void preparedatabase(DSTATE *s)
 	}
 
 	if (s->dbifcount == 0) {
-		if (importlegacydbs(s) && !s->alwaysadd) {
+		if (importlegacydbs(s) && !cfg.alwaysadd) {
 			s->dbifcount = 0;
 			return;
 		}
@@ -472,6 +471,9 @@ void processdatacache(DSTATE *s)
 			db_removeoldentries();
 			s->cleanuphour = getcurrenthour();
 		}
+		if (cfg.rescanonsave) {
+			rescandatabaseforinterfaces(s);
+		}
 		s->dodbsave = 0;
 	}
 }
@@ -721,7 +723,7 @@ void cleanremovedinterfaces(DSTATE *s)
 	if (dbifl != NULL) {
 		dbifl_iterator = dbifl;
 		while (dbifl_iterator != NULL) {
-			snprintf(errorstring, 1024, "Removing interface \"%s\" from update list.", dbifl_iterator->interface);
+			snprintf(errorstring, 1024, "Interface \"%s\" no longer in database, stopping monitoring.", dbifl_iterator->interface);
 			printe(PT_Info);
 			datacache_remove(&s->dcache, dbifl_iterator->interface);
 			if (s->dbifcount > 0) {
@@ -735,6 +737,48 @@ void cleanremovedinterfaces(DSTATE *s)
 	timeused_debug(__func__, 0);
 }
 
+void rescandatabaseforinterfaces(DSTATE *s)
+{
+	short interface_already_monitored = 0;
+	uint64_t dbifcount = s->dbifcount;
+	datacache *iterator = NULL;
+	iflist *dbifl = NULL, *dbifl_iterator = NULL;
+
+	timeused_debug(__func__, 1);
+
+	if (db_getiflist(&dbifl) > 0 && dbifl != NULL) {
+		dbifl_iterator = dbifl;
+		while (dbifl_iterator != NULL) {
+			iterator = s->dcache;
+			interface_already_monitored = 0;
+			while (iterator != NULL) {
+				if (strcmp(iterator->interface, dbifl_iterator->interface) == 0) {
+					interface_already_monitored = 1;
+					break;
+				}
+				iterator = iterator->next;
+			}
+			if (!interface_already_monitored) {
+				snprintf(errorstring, 1024, "Interface \"%s\" found from database, starting monitoring.", dbifl_iterator->interface);
+				printe(PT_Info);
+				if (!datacache_add(&s->dcache, dbifl_iterator->interface, 1)) {
+					snprintf(errorstring, 1024, "Cache memory allocation failed (%s), exiting.", strerror(errno));
+					printe(PT_Error);
+					errorexitdaemon(s, 1);
+				}
+				s->dbifcount++;
+			}
+			dbifl_iterator = dbifl_iterator->next;
+		}
+		if (s->dbifcount != dbifcount) {
+			datacache_status(&s->dcache);
+		}
+		iflistfree(&dbifl);
+	}
+
+	timeused_debug(__func__, 0);
+}
+
 void handleintsignals(DSTATE *s)
 {
 	switch (intsignal) {
@@ -747,7 +791,7 @@ void handleintsignals(DSTATE *s)
 			s->dbifcount = 0;
 			ibwflush();
 			db_close();
-			loadcfg(s->cfgfile);
+			loadcfg(s->cfgfile, CT_Daemon);
 			ibwloadcfg(s->cfgfile);
 			if (!db_open_rw(1)) {
 				snprintf(errorstring, 1024, "Opening database after SIGHUP failed (%s), exiting.", strerror(errno));
@@ -808,7 +852,7 @@ void preparedirs(DSTATE *s)
 
 void datacache_status(datacache **dc)
 {
-	char buffer[1024], bwtemp[16];
+	char buffer[1024], bwtemp[32];
 	unsigned int b = 0, count = 0;
 	uint32_t bwlimit = 0;
 	datacache *iterator = *dc;
@@ -819,11 +863,11 @@ void datacache_status(datacache **dc)
 	b = (unsigned int)strlen(buffer) + 1;
 
 	while (iterator != NULL) {
-		if ((b + strlen(iterator->interface) + 16) < 1020) {
+		if ((b + strlen(iterator->interface) + 32) < 1020) {
 			if (!ibwget(iterator->interface, &bwlimit) || bwlimit == 0) {
-				snprintf(bwtemp, 16, " (no limit) ");
+				snprintf(bwtemp, 32, " (no limit) ");
 			} else {
-				snprintf(bwtemp, 16, " (%" PRIu32 " Mbit) ", bwlimit);
+				snprintf(bwtemp, 32, " (%" PRIu32 " Mbit) ", bwlimit);
 			}
 			strcat(buffer, iterator->interface);
 			strcat(buffer, bwtemp);
