@@ -524,6 +524,53 @@ char *db_getinterfaceidin(const char *iface)
 	return result;
 }
 
+int db_setinterfacebyalias(char *iface, const char *alias, const int matchmethod)
+{
+	int rc;
+	char sql[256];
+	sqlite3_stmt *sqlstmt;
+
+	switch (matchmethod) {
+		// case sensitive
+		case 1:
+			sqlite3_snprintf(256, sql, "select name from interface where alias='%q' order by rxtotal+txtotal desc", alias);
+			break;
+		// case insensitive
+		case 2:
+			sqlite3_snprintf(256, sql, "select name from interface where alias='%q' collate nocase order by rxtotal+txtotal desc", alias);
+			break;
+		// case insensitive prefix
+		case 3:
+			sqlite3_snprintf(256, sql, "select name from interface where alias like '%q%%' collate nocase order by rxtotal+txtotal desc", alias);
+			break;
+		default:
+			return 0;
+	}
+
+	rc = sqlite3_prepare_v2(db, sql, -1, &sqlstmt, NULL);
+	if (rc != SQLITE_OK) {
+		db_errcode = rc;
+		snprintf(errorstring, 1024, "Failed to get interface alias from database (%d): %s", rc, sqlite3_errmsg(db));
+		printe(PT_Error);
+		return 0;
+	}
+
+	if (sqlite3_column_count(sqlstmt) != 1) {
+		return 0;
+	}
+
+	rc = 0;
+	if (sqlite3_step(sqlstmt) == SQLITE_ROW) {
+		if (sqlite3_column_text(sqlstmt, 0) != NULL) {
+			strncpy_nt(iface, (const char *)sqlite3_column_text(sqlstmt, 0), 32);
+			rc++;
+		}
+	}
+	sqlite3_finalize(sqlstmt);
+
+	return rc;
+}
+
 int db_setactive(const char *iface, const int active)
 {
 	char sql[64];
@@ -1171,7 +1218,7 @@ int db_getdata_range(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *
 {
 	int ret, i, rc;
 	const char *datatables[] = {"fiveminute", "hour", "day", "month", "year", "top"};
-	char sql[512], limit[64], dbegin[32], dend[44], *ifaceidin = NULL;
+	char sql[512], limit[64], dbegin[37], dend[44], *ifaceidin = NULL;
 	sqlite3_stmt *sqlstmt;
 	time_t timestamp;
 	int64_t rowid;
@@ -1197,7 +1244,11 @@ int db_getdata_range(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *
 
 	dbegin[0] = '\0';
 	if (strlen(databegin)) {
-		snprintf(dbegin, 32, "and date >= '%s'", databegin);
+		if (strcmp(databegin, "today") == 0) {
+			snprintf(dbegin, 37, "and date >= date('now'%s)", cfg.dbtzmodifier);
+		} else {
+			snprintf(dbegin, 37, "and date >= '%s'", databegin);
+		}
 	}
 
 	dend[0] = '\0';
