@@ -241,8 +241,8 @@ void livetrafficmeter(const char *iface, const int mode)
 		if (!json) {
 			if (mode == 0) {
 				/* packets per second visible */
-				snprintf(buffer, 128, "   rx: %s %*" PRIu64 " p/s", gettrafficrate(rx, LIVETIME, ratewidth), ppswidth, (uint64_t)rxp / LIVETIME);
-				snprintf(buffer2, 128, " %*s tx: %s %*" PRIu64 " p/s", paddingwidth, " ", gettrafficrate(tx, LIVETIME, ratewidth), ppswidth, (uint64_t)txp / LIVETIME);
+				snprintf(buffer, 128, "   rx: %s %*" PRIu64 " p/s", gettrafficrate(rx, LIVETIME, ratewidth), ppswidth, rxp / LIVETIME);
+				snprintf(buffer2, 128, " %*s tx: %s %*" PRIu64 " p/s", paddingwidth, " ", gettrafficrate(tx, LIVETIME, ratewidth), ppswidth, txp / LIVETIME);
 			} else {
 				/* total transfer amount visible */
 				snprintf(buffer, 128, "   rx: %s   %s", gettrafficrate(rx, LIVETIME, ratewidth), getvalue(rxtotal, 1, RT_Normal));
@@ -256,7 +256,6 @@ void livetrafficmeter(const char *iface, const int mode)
 			}
 			if (cfg.ostyle != 4) {
 				printf("%s", buffer);
-				fflush(stdout);
 			} else {
 				printf("%s\n", buffer);
 			}
@@ -283,6 +282,7 @@ void livetrafficmeter(const char *iface, const int mode)
 			printf("}}\n");
 			index++;
 		}
+		fflush(stdout);
 #ifdef CHECK_VNSTAT
 		break;
 #endif
@@ -327,7 +327,7 @@ void livetrafficmeter(const char *iface, const int mode)
 		if (timespent <= 60) {
 			printf("  time             %9" PRIu64 " seconds\n", timespent);
 		} else {
-			printf("  time               %7.2f minutes\n", timespent / (double)60);
+			printf("  time               %7.2f minutes\n", (double)timespent / (double)60);
 		}
 
 		printf("\n");
