@@ -41,6 +41,13 @@ void initparams(PARAMS *p)
 	p->xmlmode = 'a';
 	p->databegin[0] = '\0';
 	p->dataend[0] = '\0';
+
+	p->alert = 0;
+	p->alertoutput = 0;
+	p->alertexit = 0;
+	p->alerttype = 0;
+	p->alertcondition = 0;
+	p->alertlimit = 0;
 }
 
 void showhelp(PARAMS *p)
@@ -60,7 +67,9 @@ void showhelp(PARAMS *p)
 
 	printf("      --oneline [mode]             show simple parsable format\n");
 	printf("      --json [mode] [limit]        show database in json format\n");
-	printf("      --xml [mode] [limit]         show database in xml format\n\n");
+	printf("      --xml [mode] [limit]         show database in xml format\n");
+	printf("      --alert <output> <exit> <type> <condition> <limit> <unit>\n");
+	printf("                                   alert if limit is exceeded\n\n");
 
 	printf("      -tr, --traffic [time]        calculate traffic\n");
 	printf("      -l,  --live [mode]           show transfer rate in real time\n");
@@ -92,7 +101,9 @@ void showlonghelp(PARAMS *p)
 	printf("      -e,  --end <date>            set list end date\n");
 	printf("      --oneline [mode]             show simple parsable format\n");
 	printf("      --json [mode] [limit]        show database in json format\n");
-	printf("      --xml [mode] [limit]         show database in xml format\n\n");
+	printf("      --xml [mode] [limit]         show database in xml format\n");
+	printf("      --alert <output> <exit> <type> <condition> <limit> <unit>\n");
+	printf("                                   alert if limit is exceeded\n\n");
 
 	printf("Modify:\n");
 
@@ -127,7 +138,7 @@ void showlonghelp(PARAMS *p)
 	printf("See also \"man vnstat\" for longer descriptions of each option.\n");
 }
 
-void parseargs(PARAMS *p, int argc, char **argv)
+void parseargs(PARAMS *p, const int argc, char **argv)
 {
 	int currentarg;
 
@@ -420,13 +431,13 @@ void parseargs(PARAMS *p, int argc, char **argv)
 		} else if ((strcmp(argv[currentarg], "-b") == 0) || (strcmp(argv[currentarg], "--begin") == 0)) {
 			if (currentarg + 1 < argc) {
 				if (!validatedatetime(argv[currentarg + 1])) {
-					printf("Error: Invalid date format, expected YYYY-MM-DD HH:MM or YYYY-MM-DD.\n");
+					printf("Error: Invalid date format, expected YYYY-MM-DD HH:MM, YYYY-MM-DD or \"today\".\n");
 					exit(EXIT_FAILURE);
 				}
 				strncpy_nt(p->databegin, argv[currentarg + 1], 18);
 				currentarg++;
 			} else {
-				printf("Error: Date of format YYYY-MM-DD HH:MM or YYYY-MM-DD for %s missing.\n", argv[currentarg]);
+				printf("Error: Date of format YYYY-MM-DD HH:MM, YYYY-MM-DD or \"today\" for %s missing.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			}
 		} else if ((strcmp(argv[currentarg], "-e") == 0) || (strcmp(argv[currentarg], "--end") == 0)) {
@@ -485,6 +496,18 @@ void parseargs(PARAMS *p, int argc, char **argv)
 				printf("Error: Invalid or missing parameter for %s.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			}
+		} else if (strcmp(argv[currentarg], "--alert") == 0) {
+			if (currentarg + 6 >= argc) {
+				printf("Error: Invalid parameter count for %s.\n", argv[currentarg]);
+				showalerthelp();
+				exit(EXIT_FAILURE);
+			}
+			if (!parsealertargs(p, argv + currentarg)) {
+				exit(EXIT_FAILURE);
+			} else {
+				currentarg += 6;
+				p->alert = 1;
+			}
 		} else if ((strcmp(argv[currentarg], "-v") == 0) || (strcmp(argv[currentarg], "--version") == 0)) {
 			printf("vnStat %s by Teemu Toivola <tst at iki dot fi>\n", getversion());
 			exit(EXIT_SUCCESS);
@@ -514,9 +537,218 @@ void parseargs(PARAMS *p, int argc, char **argv)
 	}
 }
 
+int parsealertargs(PARAMS *p, char **argv)
+{
+	int i, u, found, currentarg = 0;
+	uint64_t alertlimit = 0, unitmultiplier = 1;
+	int32_t unitmode = cfg.unitmode;
+	const char *alerttypes[] = {"h", "hour", "hourly", "d", "day", "daily", "m", "month", "monthly", "y", "year", "yearly"};
+	const char *alertconditions[] = {"rx", "tx", "total", "rx_estimate", "tx_estimate", "total_estimate"}; // order must match that of AlertCondition in dbshow.h
+
+	for (i = 1; i <= 6; i++) {
+		if (argv[currentarg + i][0] == '-' || ishelprequest(argv[currentarg + i])) {
+			showalerthelp();
+			return 0;
+		}
+	}
+	currentarg++;
+
+	// output
+	if (!isnumeric(argv[currentarg])) {
+		printf("Error: Non-numeric output parameter \"%s\" for %s.\n", argv[currentarg], argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	p->alertoutput = (unsigned int)atoi(argv[currentarg]);
+	if (p->alertoutput > 3) {
+		printf("Error: Output parameter out of range for %s.\n", argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	if (debug) {
+		printf("Alert output: %u\n", p->alertoutput);
+	}
+	currentarg++;
+
+	// exit
+	if (!isnumeric(argv[currentarg])) {
+		printf("Error: Non-numeric exit parameter \"%s\" for %s.\n", argv[currentarg], argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	p->alertexit = (unsigned int)atoi(argv[currentarg]);
+	if (p->alertexit > 3) {
+		printf("Error: Exit parameter out of range for %s.\n", argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	if (debug) {
+		printf("Alert exit: %u\n", p->alertexit);
+	}
+
+	if (p->alertoutput == AO_No_Output && (p->alertexit == AE_Always_Exit_0 || p->alertexit == AE_Always_Exit_1)) {
+		printf("Error: Configuring %s for no output and always same exit status provides no real usability.\n", argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	currentarg++;
+
+	// type
+	found = 0;
+	for (i = 0; i < 12; i++) {
+		if (strcmp(argv[currentarg], alerttypes[i]) == 0) {
+			found = 1;
+			break;
+		}
+	}
+	if (!found) {
+		printf("Error: Invalid type parameter \"%s\" for %s.\n", argv[currentarg], argv[0]);
+		showalerthelp();
+		return 0;
+	}
+
+	switch (argv[currentarg][0]) {
+		case 'h':
+			p->alerttype = AT_Hour;
+			break;
+		case 'd':
+			p->alerttype = AT_Day;
+			break;
+		case 'm':
+			p->alerttype = AT_Month;
+			break;
+		case 'y':
+			p->alerttype = AT_Year;
+			break;
+		default:
+			return 0;
+	}
+	if (debug) {
+		printf("Alert type: %u\n", p->alerttype);
+	}
+	currentarg++;
+
+	// condition
+	found = 0;
+	for (i = 0; i < 6; i++) {
+		if (strcmp(argv[currentarg], alertconditions[i]) == 0) {
+			found = 1;
+			p->alertcondition = (unsigned int)i + 1;
+			break;
+		}
+	}
+	if (!found) {
+		printf("Error: Invalid condition parameter \"%s\" for %s.\n", argv[currentarg], argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	if (debug) {
+		printf("Alert condition: %u\n", p->alertcondition);
+	}
+	currentarg++;
+
+	if ((p->alertcondition == AC_RX_Estimate || p->alertcondition == AC_TX_Estimate || p->alertcondition == AC_Total_Estimate) &&
+		(p->alertoutput == AO_Output_On_Estimate || p->alertexit == AE_Exit_1_On_Estimate)) {
+		if (p->alertoutput == AO_Output_On_Estimate) {
+			printf("Error: Estimate conditions cannot be used in combination with output parameter \"2\".\n");
+		}
+		if (p->alertexit == AE_Exit_1_On_Estimate) {
+			printf("Error: Estimate conditions cannot be used in combination with exit parameter \"2\".\n");
+		}
+		showalerthelp();
+		return 0;
+	}
+
+	// limit
+	if (!isnumeric(argv[currentarg])) {
+		printf("Error: Limit parameter for %s must be a greater than zero integer without decimals.\n", argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	alertlimit = strtoull(argv[currentarg], (char **)NULL, 0);
+	if (alertlimit == 0) {
+		printf("Error: Invalid limit parameter \"%s\" for %s.\n", argv[currentarg], argv[0]);
+		showalerthelp();
+		return 0;
+	}
+	if (debug) {
+		printf("Alert limit: %" PRIu64 "\n", alertlimit);
+	}
+	currentarg++;
+
+	// limit unit
+	found = 0;
+	for (u = 0; u < 3; u++) {
+		cfg.unitmode = u;
+		for (i = 1; i <= UNITPREFIXCOUNT; i++) {
+			if (strcmp(argv[currentarg], getunitprefix(i)) == 0) {
+				found = 1;
+				break;
+			}
+		}
+		if (found) {
+			break;
+		}
+	}
+	cfg.unitmode = unitmode;
+	if (!found) {
+		printf("Error: Invalid limit unit parameter \"%s\" for %s.\n", argv[currentarg], argv[0]);
+		showalerthelp();
+		return 0;
+	}
+
+	unitmultiplier = getunitdivisor(u, i);
+
+	if (alertlimit > (uint64_t)(MAX64 / unitmultiplier)) {
+		printf("Error: %" PRIu64 " %s exceeds maximum supported limit of %" PRIu64 " %s.\n", alertlimit, argv[currentarg], (uint64_t)(MAX64 / unitmultiplier), argv[currentarg]);
+		return 0;
+	}
+
+	p->alertlimit = alertlimit * unitmultiplier;
+
+	if (debug) {
+		printf("Alert unit %s is %d %d = %" PRIu64 "\n", argv[currentarg], u, i, unitmultiplier);
+		printf("Alert real limit is %" PRIu64 " * %" PRIu64 " = %" PRIu64 "\n", alertlimit, unitmultiplier, p->alertlimit);
+	}
+
+	return 1;
+}
+
+void showalerthelp(void)
+{
+	printf("\n");
+	printf("Valid parameters for\n--alert <output> <exit> <type> <condition> <limit> <unit>\n\n");
+
+	printf(" <output>\n");
+	printf("    0 - no output\n");
+	printf("    1 - always show output\n");
+	printf("    2 - show output only if usage estimate exceeds limit\n");
+	printf("    3 - show output only if limit is exceeded\n\n");
+
+	printf(" <exit>\n");
+	printf("    0 - always use exit status 0\n");
+	printf("    1 - always use exit status 1\n");
+	printf("    2 - use exit status 1 if usage estimate exceeds limit\n");
+	printf("    3 - use exit status 1 if limit is exceeded\n\n");
+
+	printf(" <type>\n");
+	printf("    h, hour, hourly        d, day, daily\n");
+	printf("    m, month, monthly      y, year, yearly\n\n");
+
+	printf(" <condition>\n");
+	printf("    rx, tx, total, rx_estimate, tx_estimate, total_estimate\n\n");
+
+	printf(" <limit>\n");
+	printf("    greater than zero integer without decimals\n\n");
+
+	printf(" <unit> for <limit>\n");
+	printf("    B, KiB, MiB, GiB, TiB, PiB, EiB\n");
+	printf("    B, KB, MB, GB, TB, PB, EB\n");
+}
+
 void showstylehelp(void)
 {
-	printf(" Valid parameters for --style.:\n");
+	printf(" Valid parameters for --style:\n");
 	printf("    0 - a more narrow output\n");
 	printf("    1 - enable bar column if available\n");
 	printf("    2 - average traffic rate in summary output\n");
@@ -525,6 +757,30 @@ void showstylehelp(void)
 	printf("        and show raw values in --oneline\n");
 }
 
+void handleshowalert(PARAMS *p)
+{
+	int alert = 0;
+
+	if (!p->alert) {
+		return;
+	}
+
+	if (p->defaultiface) {
+		printf("Error: An interface needs to be explicitly specified for --alert.\n");
+		exit(EXIT_FAILURE);
+	}
+
+	validateinterface(p);
+
+	alert = showalert(p->interface, p->alertoutput, p->alertexit, p->alerttype, p->alertcondition, p->alertlimit);
+
+	if ((alert || p->alertexit == AE_Always_Exit_1) && p->alertexit != AE_Always_Exit_0) {
+		exit(EXIT_FAILURE);
+	}
+
+	exit(EXIT_SUCCESS);
+}
+
 void handleremoveinterface(PARAMS *p)
 {
 	if (!p->removeiface) {
@@ -532,7 +788,7 @@ void handleremoveinterface(PARAMS *p)
 	}
 
 	if (p->defaultiface) {
-		printf("Error: Use -i parameter to specify an interface.\n");
+		printf("Error: An interface needs to be explicitly specified for --remove.\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -574,7 +830,7 @@ void handlerenameinterface(PARAMS *p)
 	}
 
 	if (p->defaultiface) {
-		printf("Error: Use -i parameter to specify an interface.\n");
+		printf("Error: An interface needs to be explicitly specified for --rename.\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -625,7 +881,7 @@ void handleaddinterface(PARAMS *p)
 	}
 
 	if (p->defaultiface) {
-		printf("Error: Use -i parameter to specify an interface.\n");
+		printf("Error: An interface needs to be explicitly specified for --add.\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -683,7 +939,7 @@ void handlesetalias(PARAMS *p)
 	}
 
 	if (p->defaultiface) {
-		printf("Error: Use -i parameter to specify an interface.\n");
+		printf("Error: An interface needs to be explicitly specified for --setalias.\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -726,7 +982,7 @@ void handleshowdata(PARAMS *p)
 		return;
 	}
 
-	/* show all interfaces if -i isn't specified */
+	/* show all interfaces if none is explicitly specified */
 	if (p->dbifcount == 0) {
 		p->query = 0;
 	} else if ((cfg.qmode == 0 || cfg.qmode == 8 || cfg.qmode == 10) && (p->dbifcount > 1)) {
@@ -779,14 +1035,7 @@ void handleshowdata(PARAMS *p)
 
 void showoneinterface(PARAMS *p)
 {
-	if (!db_getinterfacecountbyname(p->interface)) {
-		if (strchr(p->interface, '+') == NULL) {
-			printf("Error: Interface \"%s\" not found in database.\n", p->interface);
-		} else {
-			printf("Error: Not all requested interfaces found in database or given interfaces aren't unique.\n");
-		}
-		exit(EXIT_FAILURE);
-	}
+	validateinterface(p);
 
 	if (cfg.qmode == 5) {
 		if (cfg.ostyle != 0) {
@@ -834,9 +1083,9 @@ void handletrafficmeters(PARAMS *p)
 			printf("Error: Configured default interface \"%s\" isn't available.\n\n", p->interface);
 			if (strlen(cfg.cfgfile)) {
 				printf("Update \"Interface\" keyword value in configuration file \"%s\" to change ", cfg.cfgfile);
-				printf("the default interface or give an alternative interface using the -i parameter.\n\n");
+				printf("the default interface or give an alternative interface with or without the -i parameter.\n\n");
 			} else {
-				printf("An alternative interface can be given using the -i parameter.\n\n");
+				printf("An alternative interface can be given with or without the -i parameter.\n\n");
 			}
 			printf("The following interfaces are currently available:\n    %s\n", p->ifacelist);
 
@@ -1002,3 +1251,33 @@ void showdbiflist(const int parseable)
 	iflistfree(&dbifl);
 	db_close();
 }
+
+void validateinterface(PARAMS *p)
+{
+	int i, found = 0;
+
+	timeused_debug(__func__, 1);
+
+	if (!db_getinterfacecountbyname(p->interface)) {
+		if (strchr(p->interface, '+') == NULL) {
+			for (i = 1; i <= cfg.ifacematchmethod; i++) {
+				found = db_setinterfacebyalias(p->interface, p->interface, i);
+				if (found) {
+					if (debug) {
+						printf("Found \"%s\" with method %d\n", p->interface, i);
+					}
+					break;
+				}
+			}
+			if (!found) {
+				printf("Error: No interface matching \"%s\" found in database.\n", p->interface);
+				exit(EXIT_FAILURE);
+			}
+		} else {
+			printf("Error: Not all requested interfaces found in database or given interfaces aren't unique.\n");
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	timeused_debug(__func__, 0);
+}
