@@ -10,12 +10,18 @@ typedef struct {
 	char interface[32], alias[32], newifname[32], filename[512];
 	char definterface[32], cfgfile[512], *ifacelist, jsonmode, xmlmode;
 	char databegin[18], dataend[18];
+	unsigned int alert, alertoutput, alertexit, alerttype, alertcondition;
+	uint64_t alertlimit;
 } PARAMS;
 
 void initparams(PARAMS *p);
 void showhelp(PARAMS *p);
 void showlonghelp(PARAMS *p);
-void parseargs(PARAMS *p, int argc, char **argv);
+void parseargs(PARAMS *p, const int argc, char **argv);
+int parsealertargs(PARAMS *p, char **argv);
+void showalerthelp(void);
+void showstylehelp(void);
+void handleshowalert(PARAMS *p);
 void handleremoveinterface(PARAMS *p);
 void handlerenameinterface(PARAMS *p);
 void handleaddinterface(PARAMS *p);
@@ -24,5 +30,8 @@ void handleshowdata(PARAMS *p);
 void showoneinterface(PARAMS *p);
 void handletrafficmeters(PARAMS *p);
 void handleifselection(PARAMS *p);
+void showiflist(const int parseable);
+void showdbiflist(const int parseable);
+void validateinterface(PARAMS *p);
 
 #endif
