@@ -1,5 +1,5 @@
 /*
-vnStat image output - Copyright (C) 2007-2020 Teemu Toivola <tst@iki.fi>
+vnStat image output - Copyright (C) 2007-2022 Teemu Toivola <tst@iki.fi>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ int main(int argc, char *argv[])
 		for (currentarg = 1; currentarg < argc; currentarg++) {
 			if ((strcmp(argv[currentarg], "-D") == 0) || (strcmp(argv[currentarg], "--debug") == 0)) {
 				debug = 1;
-				printf("Debug enabled, vnstati %s\n", VERSION);
+				printf("Debug enabled, vnstati %s, LibGD %d.%d.%d\n", VERSION, GD_MAJOR_VERSION, GD_MINOR_VERSION, GD_RELEASE_VERSION);
 			} else if (strcmp(argv[currentarg], "--config") == 0) {
 				if (currentarg + 1 < argc) {
 					strncpy_nt(p.cfgfile, argv[currentarg + 1], 512);
@@ -53,7 +53,7 @@ int main(int argc, char *argv[])
 	}
 
 	/* load config if available */
-	if (!loadcfg(p.cfgfile)) {
+	if (!loadcfg(p.cfgfile, CT_Image)) {
 		return 1;
 	}
 	cfg.qmode = 0;
@@ -64,15 +64,23 @@ int main(int argc, char *argv[])
 
 	parseargs(&p, &ic, argc, argv);
 	validateinput(&p);
+	validateoutput(&p);
 	handlecaching(&p, &ic);
 	handledatabase(&p, &ic);
-	openoutput(&p);
 
 	if (debug)
 		printf("Qmode: %d\n", cfg.qmode);
 
 	drawimage(&ic);
 	db_close();
+
+	if (ic.im == NULL) {
+		return 1;
+	}
+
+#if HAVE_DECL_GD_NEAREST_NEIGHBOUR
+	scaleimage(&ic);
+#endif
 	writeoutput(&p, &ic);
 
 	if (debug)
@@ -100,40 +108,55 @@ void showihelp(IPARAMS *p)
 {
 	printf("vnStat image output %s by Teemu Toivola <tst at iki dot fi>\n\n", getversion());
 
-	printf("      -5,  --fiveminutes [limit]   output 5 minutes\n");
-	printf("      -h,  --hours [limit]         output hours\n");
-	printf("      -hg, --hoursgraph            output hours graph\n");
-	printf("      -d,  --days [limit]          output days\n");
-	printf("      -m,  --months [limit]        output months\n");
-	printf("      -y,  --years [limit]         output years\n");
-	printf("      -t,  --top [limit]           output top days\n");
-	printf("      -s,  --summary               output summary\n");
-	printf("      -hs, --hsummary              output horizontal summary with hours\n");
-	printf("      -vs, --vsummary              output vertical summary with hours\n");
-	printf("      -nh, --noheader              remove header from output\n");
-	printf("      -ne, --noedge                remove edge from output\n");
-	printf("      -nl, --nolegend              remove legend from output\n");
-	printf("      -ru, --rateunit [mode]       swap configured rate unit\n");
-	printf("      -o,  --output <file>         select output filename\n");
-	printf("      -c,  --cache <minutes>       update output only when too old\n");
-	printf("      -i,  --iface <interface>     select interface");
+	printf("      -5,  --fiveminutes [limit]         output 5 minutes\n");
+	printf("      -5g, --fivegraph [limit] [height]  output 5 minutes graph\n");
+	printf("      -h,  --hours [limit]               output hours\n");
+	printf("      -hg, --hoursgraph                  output hours graph\n");
+	printf("      -d,  --days [limit]                output days\n");
+	printf("      -m,  --months [limit]              output months\n");
+	printf("      -y,  --years [limit]               output years\n");
+	printf("      -t,  --top [limit]                 output top days\n");
+	printf("      -s,  --summary                     output summary\n");
+	printf("      -hs, --hsummary [graph]            output horizontal summary with graph\n");
+	printf("      -vs, --vsummary [graph]            output vertical summary with graph\n\n");
+
+	printf("      -nh, --noheader                    remove header from output\n");
+	printf("      -ne, --noedge                      remove edge from output\n");
+	printf("      -nl, --nolegend                    remove legend from output\n");
+	printf("      -ru, --rateunit [mode]             swap configured rate unit\n");
+	printf("      -S,  --small                       use small fonts");
+	if (!cfg.largefonts) {
+		printf(" (default)");
+	}
+	printf("\n");
+	printf("      -L,  --large                       use large fonts");
+	if (cfg.largefonts) {
+		printf(" (default)");
+	}
+	printf("\n");
+	printf("      -o,  --output <file>               select output filename\n");
+	printf("      -c,  --cache <minutes>             update output only when too old\n");
+	printf("      -i,  --iface <interface>           select interface");
 	if (strlen(p->interface)) {
 		printf(" (default: %s)", p->interface);
 	}
 	printf("\n");
-	printf("      -b,  --begin <date>          set list begin date\n");
-	printf("      -e,  --end <date>            set list end date\n");
-	printf("      -?,  --help                  this help\n");
-	printf("      -D,  --debug                 show some additional debug information\n");
-	printf("      -v,  --version               show version\n");
-	printf("      --limit <limit>              set list entry limit\n");
-	printf("      --dbdir <directory>          select database directory\n");
-	printf("      --style <mode>               select output style (0-3)\n");
-	printf("      --locale <locale>            set locale\n");
-	printf("      --config <config file>       select config file\n");
-	printf("      --altdate                    use alternative date location\n");
-	printf("      --headertext <text>          specify header text string\n");
-	printf("      --transparent [enabled]      toggle background transparency\n\n");
+	printf("      -b,  --begin <date>                set list begin date\n");
+	printf("      -e,  --end <date>                  set list end date\n");
+	printf("      -?,  --help                        this help\n");
+	printf("      -D,  --debug                       show some additional debug information\n");
+	printf("      -v,  --version                     show version\n");
+	printf("      --limit <limit>                    set list entry limit\n");
+	printf("      --dbdir <directory>                select database directory\n");
+	printf("      --style <mode>                     select output style (0-3)\n");
+	printf("      --locale <locale>                  set locale\n");
+	printf("      --config <config file>             select config file\n");
+	printf("      --altdate                          use alternative date location\n");
+	printf("      --headertext <text>                specify header text string\n");
+#if HAVE_DECL_GD_NEAREST_NEIGHBOUR
+	printf("      --scale <percent>                  change image size by scaling it\n");
+#endif
+	printf("      --transparent [enabled]            toggle background transparency\n\n");
 
 	printf("See also \"man vnstati\".\n");
 }
@@ -148,7 +171,7 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 			printf("arg %d: \"%s\"\n", currentarg, argv[currentarg]);
 		if ((strcmp(argv[currentarg], "-?") == 0) || (strcmp(argv[currentarg], "--help")) == 0) {
 			p->help = 1;
-		} else if ((strcmp(argv[currentarg], "-i") == 0) || (strcmp(argv[currentarg], "--iface")) == 0) {
+		} else if ((strcmp(argv[currentarg], "-i") == 0) || (strcmp(argv[currentarg], "--iface")) == 0 || (strcmp(argv[currentarg], "--interface") == 0)) {
 			if (currentarg + 1 < argc) {
 				if (strlen(argv[currentarg + 1]) > 31) {
 					printf("Error: Interface name is limited to 31 characters.\n");
@@ -159,7 +182,7 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 					printf("Used interface: \"%s\"\n", p->interface);
 				currentarg++;
 			} else {
-				printf("Error: Interface for -i missing.\n");
+				printf("Error: Interface for %s missing.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			}
 		} else if ((strcmp(argv[currentarg], "-o") == 0) || (strcmp(argv[currentarg], "--output")) == 0) {
@@ -169,7 +192,7 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 					printf("Output file: \"%s\"\n", p->filename);
 				currentarg++;
 			} else {
-				printf("Error: Filename for -o missing.\n");
+				printf("Error: Filename for %s missing.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			}
 		} else if ((strcmp(argv[currentarg], "-c") == 0) || (strcmp(argv[currentarg], "--cache")) == 0) {
@@ -179,7 +202,7 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 					printf("Cache time: %d minutes\n", p->cache);
 				currentarg++;
 			} else {
-				printf("Error: Parameter for -c missing or invalid.\n");
+				printf("Error: Parameter for %s missing or invalid.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			}
 		} else if ((strcmp(argv[currentarg], "--style")) == 0) {
@@ -196,6 +219,25 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 				printf("Error: Style parameter for --style missing.\n");
 				exit(EXIT_FAILURE);
 			}
+		} else if ((strcmp(argv[currentarg], "--scale")) == 0) {
+#if HAVE_DECL_GD_NEAREST_NEIGHBOUR
+			if (currentarg + 1 < argc && isdigit(argv[currentarg + 1][0])) {
+				cfg.imagescale = atoi(argv[currentarg + 1]);
+				if (cfg.imagescale > 500 || cfg.imagescale < 50) {
+					printf("Error: Invalid parameter \"%d\" for --scale. Supported value range: 50 <= N <= 500\n", cfg.imagescale);
+					exit(EXIT_FAILURE);
+				}
+				if (debug)
+					printf("Scale changed: %d\n", cfg.imagescale);
+				currentarg++;
+			} else {
+				printf("Error: Percent parameter for --scale missing.\n");
+				exit(EXIT_FAILURE);
+			}
+#else
+			printf("Error: Function needed by --scale is not available in used LibGD %d.%d.%d.\n", GD_MAJOR_VERSION, GD_MINOR_VERSION, GD_RELEASE_VERSION);
+			exit(EXIT_FAILURE);
+#endif
 		} else if ((strcmp(argv[currentarg], "--transparent")) == 0) {
 			if (currentarg + 1 < argc && isdigit(argv[currentarg + 1][0])) {
 				cfg.transbg = atoi(argv[currentarg + 1]);
@@ -246,6 +288,10 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 			}
 		} else if (strcmp(argv[currentarg], "--altdate") == 0) {
 			ic->altdate = 1;
+		} else if ((strcmp(argv[currentarg], "-L") == 0) || (strcmp(argv[currentarg], "--large")) == 0) {
+			cfg.largefonts = 1;
+		} else if ((strcmp(argv[currentarg], "-S") == 0) || (strcmp(argv[currentarg], "--small")) == 0) {
+			cfg.largefonts = 0;
 		} else if ((strcmp(argv[currentarg], "-D") == 0) || (strcmp(argv[currentarg], "--debug")) == 0) {
 			debug = 1;
 		} else if ((strcmp(argv[currentarg], "-d") == 0) || (strcmp(argv[currentarg], "--days")) == 0) {
@@ -310,12 +356,75 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 			}
 		} else if ((strcmp(argv[currentarg], "-s") == 0) || (strcmp(argv[currentarg], "--summary")) == 0) {
 			cfg.qmode = 5;
-		} else if ((strcmp(argv[currentarg], "-hg") == 0) || (strcmp(argv[currentarg], "--hoursgraph")) == 0) {
+		} else if ((strcmp(argv[currentarg], "-hg") == 0) || (strcmp(argv[currentarg], "--hoursgraph") == 0) || (strcmp(argv[currentarg], "--hourlygraph")) == 0) {
 			cfg.qmode = 7;
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode selection \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid modes for %s:\n", argv[currentarg]);
+					printf("    0 - 24 hour sliding windows\n");
+					printf("    1 - graph start from midnight\n");
+					exit(EXIT_FAILURE);
+				}
+				cfg.hourlygmode = atoi(argv[currentarg + 1]);
+				if (debug)
+					printf("Hourly graph mode changed: %d\n", cfg.hourlygmode);
+				currentarg++;
+			}
+		} else if ((strcmp(argv[currentarg], "-5g") == 0) || (strcmp(argv[currentarg], "--fivegraph")) == 0) {
+			cfg.qmode = 10;
+			if (currentarg + 1 < argc && isdigit(argv[currentarg + 1][0])) {
+				cfg.fivegresultcount = atoi(argv[currentarg + 1]);
+				if (cfg.fivegresultcount < FIVEGMINRESULTCOUNT) {
+					printf("Error: Invalid limit parameter \"%s\" for %s. A value equal or over %d is expected.\n", argv[currentarg + 1], argv[currentarg], FIVEGMINRESULTCOUNT);
+					exit(EXIT_FAILURE);
+				} else if (cfg.fivegresultcount > cfg.fiveminutehours * 12) {
+					printf("Error: Invalid limit parameter \"%s\" for %s. Value cannot be larger than configured data retention (5MinuteHours %d * 12 = %d).\n", argv[currentarg + 1], argv[currentarg], cfg.fiveminutehours, cfg.fiveminutehours * 12);
+					exit(EXIT_FAILURE);
+				}
+				currentarg++;
+			}
+			if (currentarg + 1 < argc && isdigit(argv[currentarg + 1][0])) {
+				cfg.fivegheight = atoi(argv[currentarg + 1]);
+				if (cfg.fivegheight < FIVEGMINHEIGHT) {
+					printf("Error: Invalid height parameter \"%s\" for %s. A value equal or over %d is expected.\n", argv[currentarg + 1], argv[currentarg], FIVEGMINHEIGHT);
+					exit(EXIT_FAILURE);
+				}
+				currentarg++;
+			}
 		} else if ((strcmp(argv[currentarg], "-hs") == 0) || (strcmp(argv[currentarg], "--hsummary")) == 0) {
 			cfg.qmode = 51;
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid graph selection \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid graphs for %s:\n", argv[currentarg]);
+					printf("    0 - hours\n");
+					printf("    1 - 5 minutes\n");
+					exit(EXIT_FAILURE);
+				}
+				cfg.summarygraph = atoi(argv[currentarg + 1]);
+				if (debug)
+					printf("Summary graph changed: %d\n", cfg.summarygraph);
+				currentarg++;
+			}
 		} else if ((strcmp(argv[currentarg], "-vs") == 0) || (strcmp(argv[currentarg], "--vsummary")) == 0) {
 			cfg.qmode = 52;
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid graph selection \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid graphs for %s:\n", argv[currentarg]);
+					printf("    0 - hours\n");
+					printf("    1 - 5 minutes\n");
+					exit(EXIT_FAILURE);
+				}
+				cfg.summarygraph = atoi(argv[currentarg + 1]);
+				if (debug)
+					printf("Summary graph changed: %d\n", cfg.summarygraph);
+				currentarg++;
+			}
 		} else if ((strcmp(argv[currentarg], "-nh") == 0) || (strcmp(argv[currentarg], "--noheader")) == 0) {
 			ic->showheader = 0;
 		} else if ((strcmp(argv[currentarg], "-ne") == 0) || (strcmp(argv[currentarg], "--noedge")) == 0) {
@@ -323,15 +432,16 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 		} else if ((strcmp(argv[currentarg], "-nl") == 0) || (strcmp(argv[currentarg], "--nolegend")) == 0) {
 			ic->showlegend = 0;
 		} else if ((strcmp(argv[currentarg], "-ru") == 0) || (strcmp(argv[currentarg], "--rateunit")) == 0) {
-			if (currentarg + 1 < argc && isdigit(argv[currentarg + 1][0])) {
-				cfg.rateunit = atoi(argv[currentarg + 1]);
-				if (cfg.rateunit > 1 || cfg.rateunit < 0) {
-					printf("Error: Invalid parameter \"%d\" for --rateunit.\n", cfg.rateunit);
-					printf(" Valid parameters:\n");
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for %s:\n", argv[currentarg]);
 					printf("    0 - bytes\n");
 					printf("    1 - bits\n");
 					exit(EXIT_FAILURE);
 				}
+				cfg.rateunit = atoi(argv[currentarg + 1]);
 				if (debug)
 					printf("Rateunit changed: %d\n", cfg.rateunit);
 				currentarg++;
@@ -343,13 +453,13 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 		} else if ((strcmp(argv[currentarg], "-b") == 0) || (strcmp(argv[currentarg], "--begin") == 0)) {
 			if (currentarg + 1 < argc) {
 				if (!validatedatetime(argv[currentarg + 1])) {
-					printf("Error: Invalid date format, expected YYYY-MM-DD HH:MM or YYYY-MM-DD.\n");
+					printf("Error: Invalid date format, expected YYYY-MM-DD HH:MM, YYYY-MM-DD or \"today\".\n");
 					exit(EXIT_FAILURE);
 				}
 				strncpy_nt(ic->databegin, argv[currentarg + 1], 18);
 				currentarg++;
 			} else {
-				printf("Error: Date of format YYYY-MM-DD HH:MM or YYYY-MM-DD for %s missing.\n", argv[currentarg]);
+				printf("Error: Date of format YYYY-MM-DD HH:MM, YYYY-MM-DD or \"today\" for %s missing.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			}
 		} else if ((strcmp(argv[currentarg], "-e") == 0) || (strcmp(argv[currentarg], "--end") == 0)) {
@@ -377,11 +487,21 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 				exit(EXIT_FAILURE);
 			}
 		} else if ((strcmp(argv[currentarg], "-v") == 0) || (strcmp(argv[currentarg], "--version")) == 0) {
-			printf("vnStat image output %s by Teemu Toivola <tst at iki dot fi>\n", getversion());
+			printf("vnStat image output %s by Teemu Toivola <tst at iki dot fi> (LibGD %d.%d.%d)\n", getversion(), GD_MAJOR_VERSION, GD_MINOR_VERSION, GD_RELEASE_VERSION);
 			exit(EXIT_SUCCESS);
 		} else {
-			printf("Unknown arg \"%s\". Use --help for help.\n", argv[currentarg]);
-			exit(EXIT_FAILURE);
+			if (argv[currentarg][0] == '-') {
+				printf("Unknown parameter \"%s\". Use --help for help.\n", argv[currentarg]);
+				exit(EXIT_FAILURE);
+			} else {
+				if (strlen(argv[currentarg]) > 31) {
+					printf("Error: Interface name is limited to 31 characters.\n");
+					exit(EXIT_FAILURE);
+				}
+				strncpy_nt(p->interface, argv[currentarg], 32);
+				if (debug)
+					printf("Used interface: \"%s\"\n", p->interface);
+			}
 		}
 	}
 
@@ -393,12 +513,22 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 	if (p->limit != -1) {
 		cfg.listfivemins = cfg.listhours = cfg.listdays = cfg.listmonths = cfg.listyears = cfg.listtop = cfg.listjsonxml = p->limit;
 	}
+
+	if (cfg.largefonts) {
+		ic->font = gdFontGetLarge();
+		ic->lineheight = 16;
+		ic->large = 1;
+	} else {
+		ic->font = gdFontGetSmall();
+		ic->lineheight = 12;
+		ic->large = 0;
+	}
 }
 
 void validateinput(IPARAMS *p)
 {
 	if (!cfg.qmode || !strlen(p->filename)) {
-		printf("At least output mode and file parameter needs to be given.\n");
+		printf("At least output mode and file parameter needs to be given. ");
 		printf("Use -? or --help for getting short help.\n");
 		exit(EXIT_FAILURE);
 	}
@@ -429,6 +559,7 @@ void handlecaching(IPARAMS *p, IMAGECONTENT *ic)
 
 void handledatabase(IPARAMS *p, IMAGECONTENT *ic)
 {
+	int i, found = 0;
 	iflist *dbifl = NULL;
 
 	if (!db_open_ro()) {
@@ -438,11 +569,23 @@ void handledatabase(IPARAMS *p, IMAGECONTENT *ic)
 	if (strlen(p->interface)) {
 		if (!db_getinterfacecountbyname(p->interface)) {
 			if (strchr(p->interface, '+') == NULL) {
-				printf("Error: Interface \"%s\" not found in database.\n", p->interface);
+				for (i = 1; i <= cfg.ifacematchmethod; i++) {
+					found = db_setinterfacebyalias(p->interface, p->interface, i);
+					if (found) {
+						if (debug) {
+							printf("Found \"%s\" with method %d\n", p->interface, i);
+						}
+						break;
+					}
+				}
+				if (!found) {
+					printf("Error: No interface matching \"%s\" found in database.\n", p->interface);
+					exit(EXIT_FAILURE);
+				}
 			} else {
 				printf("Error: Not all requested interfaces found in database or given interfaces aren't unique.\n");
+				exit(EXIT_FAILURE);
 			}
-			exit(EXIT_FAILURE);
 		}
 	} else {
 		if (db_getiflist_sorted(&dbifl, 1) <= 0) {
@@ -455,30 +598,80 @@ void handledatabase(IPARAMS *p, IMAGECONTENT *ic)
 			printf("Automatically selected interface: \"%s\"\n", p->interface);
 	}
 	if (!db_getinterfaceinfo(p->interface, &ic->interface)) {
-		printf("Error: Failed to fetch interface \"%s\" info from database.\n", p->interface);
+		printf("Error: Failed to fetch interface \"%s\" details from database.\n", p->interface);
 		exit(EXIT_FAILURE);
 	}
 }
 
-void openoutput(IPARAMS *p)
+void validateoutput(IPARAMS *p)
 {
-	if (p->filename[0] != '-') {
-		if ((p->pngout = fopen(p->filename, "w")) == NULL) {
-			printf("Error: Opening file \"%s\" for output failed: %s\n", p->filename, strerror(errno));
-			exit(EXIT_FAILURE);
-		}
-	} else {
-		/* output to stdout */
-		if ((p->pngout = fdopen(1, "w")) == NULL) {
-			printf("Error: Opening stdout for output failed: %s\n", strerror(errno));
+#if HAVE_DECL_GDIMAGEFILE
+	/* not output to stdout */
+	if (!(strlen(p->filename) == 1 && p->filename[0] == '-')) {
+		if (!gdSupportsFileType(p->filename, 1)) {
+			printf("Error: Image format file extension for \"%s\" is not supported or recognized\n\n", p->filename);
+			showsupportedfileextensions();
 			exit(EXIT_FAILURE);
 		}
 	}
+#else
+	/* show warning if given filename doesn't end with .png when gdImageFile() isn't available */
+	if (!(strlen(p->filename) >= 4 && strcmp(p->filename + strlen(p->filename) - 4, ".png") == 0)) {
+		printf("Warning: Image format selection based on file extension is not available in used LibGD %d.%d.%d, \"%s\" will be written as png.\n", GD_MAJOR_VERSION, GD_MINOR_VERSION, GD_RELEASE_VERSION, p->filename);
+	}
+#endif
 }
 
 void writeoutput(IPARAMS *p, IMAGECONTENT *ic)
 {
-	gdImagePng(ic->im, p->pngout);
-	fclose(p->pngout);
+	if (ic->im == NULL) {
+		return;
+	}
+
+	/* output to stdout is always png */
+	if (strlen(p->filename) == 1 && p->filename[0] == '-') {
+		if ((p->pngout = fdopen(1, "w")) == NULL) {
+			printf("Error: Opening stdout for output failed: %s\n", strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+		gdImagePng(ic->im, p->pngout);
+		fclose(p->pngout);
+	} else {
+#if HAVE_DECL_GDIMAGEFILE
+		/* avoid "Palette image not supported by webp" */
+		if (strlen(p->filename) >= 5 && strcmp(p->filename + strlen(p->filename) - 5, ".webp") == 0) {
+			gdImagePaletteToTrueColor(ic->im);
+		}
+		if (!gdImageFile(ic->im, p->filename)) {
+			printf("Error: Writing output to \"%s\" failed: %s\n", p->filename, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+#else
+		if ((p->pngout = fopen(p->filename, "w")) == NULL) {
+			printf("Error: Opening file \"%s\" for output failed: %s\n", p->filename, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+		gdImagePng(ic->im, p->pngout);
+		fclose(p->pngout);
+#endif
+	}
+
 	gdImageDestroy(ic->im);
 }
+
+#if HAVE_DECL_GDIMAGEFILE
+void showsupportedfileextensions(void)
+{
+	int i;
+	const char *extensions[] = {".avif", ".bmp", ".gif", ".heif", ".heix", ".jpeg", ".jpg", ".png", ".tga", ".tif", ".tiff", ".wbmp", ".webp", ".xbm", ".xpm"};
+
+	printf("Supported image format file extensions in current environment:\n");
+
+	for (i = 0; i < 15; i++) {
+		if (gdSupportsFileType(extensions[i], 1)) {
+			printf("%s ", extensions[i]);
+		}
+	}
+	printf("\n");
+}
+#endif
