@@ -14,7 +14,10 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv);
 void validateinput(IPARAMS *p);
 void handlecaching(IPARAMS *p, IMAGECONTENT *ic);
 void handledatabase(IPARAMS *p, IMAGECONTENT *ic);
-void openoutput(IPARAMS *p);
+void validateoutput(IPARAMS *p);
 void writeoutput(IPARAMS *p, IMAGECONTENT *ic);
+#if HAVE_DECL_GDIMAGEFILE
+void showsupportedfileextensions(void);
+#endif
 
 #endif
