@@ -4,6 +4,7 @@
 #include "dbsql.h"
 #include "cfg.h"
 #include "image.h"
+#include "image_support.h"
 
 START_TEST(initimagecontent_does_not_crash)
 {
@@ -12,12 +13,10 @@ START_TEST(initimagecontent_does_not_crash)
 }
 END_TEST
 
-START_TEST(colorinit_does_not_crash)
+START_TEST(imageinit_does_not_crash)
 {
 	IMAGECONTENT ic;
-	defaultcfg();
-	ic.im = gdImageCreate(2, 2);
-	colorinit(&ic);
+	imageinit(&ic, 2, 2);
 	gdImageDestroy(ic.im);
 }
 END_TEST
@@ -25,10 +24,8 @@ END_TEST
 START_TEST(layoutinit_does_not_crash)
 {
 	IMAGECONTENT ic;
-	defaultcfg();
 	initimagecontent(&ic);
-	ic.im = gdImageCreate(640, 480);
-	colorinit(&ic);
+	imageinit(&ic, 640, 480);
 	ic.interface.updated = time(NULL);
 	layoutinit(&ic, "testing 123", 640, 480);
 	gdImageDestroy(ic.im);
@@ -231,41 +228,45 @@ END_TEST
 /* in order to test the right thing */
 char *hourly_imagescale_logic(const uint64_t max, const int rate)
 {
-	int i, step, s, prev = 0;
+	// int i, prev = 0;
+	int step = 0, s, extray = 0;
 	uint64_t scaleunit;
 
 	scaleunit = getscale(max, rate);
-	if (max / scaleunit > 4) {
-		step = 2;
-	} else {
-		step = 1;
-	}
 
-	for (i = step; (uint64_t)(scaleunit * (unsigned int)i) <= max; i = i + step) {
-		s = (int)(121 * ((scaleunit * (unsigned int)i) / (float)max));
-		prev = s;
+	s = (int)lrint(((double)scaleunit / (double)max) * (124 + extray));
+	if (s == 0) {
+		s = 1;
 	}
-
-	s = (int)(121 * ((scaleunit * (unsigned int)i) / (float)max));
-	if (((s + prev) / 2) <= 128) {
-		;
-	} else {
-		i = i - step;
+	while (s * step < SCALEMINPIXELS) {
+		step++;
 	}
-
+	/*
+	for (i = step; i * s <= (124 + extray + 4); i = i + step) {
+		prev = i * s;
+	}
+	*/
 	/* debug for times when things don't appear to make sense */
-	/*printf("\nmax:        %"PRIu64"\n", max);
+	/*printf("\nrate:       %d\n", rate);
+	printf("lines:      %d\n", i-1);
+	printf("max:        %"PRIu64"\n", max);
 	printf("scaleunit:  %"PRIu64"\n", scaleunit);
 	printf("old 2.0:    %"PRIu64" (i: %d, step: %d)\n", scaleunit * (i - step), i, step);
-	printf("now:        %"PRIu64" (i: %d, step: %d)\n", scaleunit * i, i, step);*/
+	printf("old 2.6:    %"PRIu64" (i: %d, step: %d)\n", scaleunit * i, i, step);
+	printf("now:        %"PRIu64" (i: %d, step: %d)\n", scaleunit * step, i, step);
+	fflush(stdout);*/
 
-	return getimagescale(scaleunit * (unsigned int)i, rate);
+	return getimagescale(scaleunit * (unsigned int)step, rate);
 }
 
 START_TEST(hourly_imagescaling_normal)
 {
 	char *unittext;
 
+	cfg.unitmode = 0;
+	cfg.rateunit = 1;
+	cfg.rateunitmode = 1;
+
 	unittext = hourly_imagescale_logic(1, 0);
 	ck_assert_str_eq(unittext, "B");
 
@@ -276,7 +277,7 @@ START_TEST(hourly_imagescaling_normal)
 	ck_assert_str_eq(unittext, "B");
 
 	unittext = hourly_imagescale_logic(1000, 0);
-	ck_assert_str_eq(unittext, "KiB");
+	ck_assert_str_eq(unittext, "B");
 
 	unittext = hourly_imagescale_logic(1024, 0);
 	ck_assert_str_eq(unittext, "KiB");
@@ -288,7 +289,7 @@ START_TEST(hourly_imagescaling_normal)
 	ck_assert_str_eq(unittext, "KiB");
 
 	unittext = hourly_imagescale_logic(1024000, 0);
-	ck_assert_str_eq(unittext, "MiB");
+	ck_assert_str_eq(unittext, "KiB");
 
 	unittext = hourly_imagescale_logic(1300000, 0);
 	ck_assert_str_eq(unittext, "MiB");
@@ -311,6 +312,7 @@ START_TEST(hourly_imagescaling_rate_1024)
 {
 	char *unittext;
 
+	cfg.unitmode = 0;
 	cfg.rateunit = 0;
 	cfg.rateunitmode = 0;
 
@@ -324,7 +326,7 @@ START_TEST(hourly_imagescaling_rate_1024)
 	ck_assert_str_eq(unittext, "B/s");
 
 	unittext = hourly_imagescale_logic(1000, 1);
-	ck_assert_str_eq(unittext, "KiB/s");
+	ck_assert_str_eq(unittext, "B/s");
 
 	unittext = hourly_imagescale_logic(1024, 1);
 	ck_assert_str_eq(unittext, "KiB/s");
@@ -336,7 +338,7 @@ START_TEST(hourly_imagescaling_rate_1024)
 	ck_assert_str_eq(unittext, "KiB/s");
 
 	unittext = hourly_imagescale_logic(1024000, 1);
-	ck_assert_str_eq(unittext, "MiB/s");
+	ck_assert_str_eq(unittext, "KiB/s");
 
 	unittext = hourly_imagescale_logic(1300000, 1);
 	ck_assert_str_eq(unittext, "MiB/s");
@@ -359,6 +361,7 @@ START_TEST(hourly_imagescaling_rate_1000)
 {
 	char *unittext;
 
+	cfg.unitmode = 0;
 	cfg.rateunit = 1;
 	cfg.rateunitmode = 1;
 
@@ -412,186 +415,185 @@ START_TEST(libgd_output_comparison)
 	x = 1060;
 	y = 420;
 
-	defaultcfg();
 	initimagecontent(&ic);
-	ic.im = gdImageCreate(x, y);
-	colorinit(&ic);
+	imageinit(&ic, x, y);
 	ic.interface.updated = (time_t)get_timestamp(2001, 2, 3, 4, 5);
 	layoutinit(&ic, "vnstati libgd output comparison", x, y);
 
 	pngout = fopen("vnstati_libgd_comparison_check.png", "w");
 	ck_assert_ptr_ne(pngout, NULL);
 
-	drawlegend(&ic, 40, 30);
+	drawlegend(&ic, 40, 30, 0);
+	drawlegend(&ic, 240, 30, 1);
 
 	/* line 1 */
 	x = 40;
 	y = 80;
 	gdImageStringUp(ic.im, gdFontGetSmall(), 1, y + 105, (unsigned char *)"libgd bug workaround", ic.ctext);
-	drawdonut(&ic, x, y, (float)0, (float)0);
-	drawdonut(&ic, x + 55, y, (float)50, (float)50);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)0, (float)0, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)50, (float)50, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"0/0 - 50/50", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)100, (float)0);
-	drawdonut(&ic, x + 55, y, (float)0, (float)100);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)100, (float)0, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)0, (float)100, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"100/0 - 0/100", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)60, (float)40);
-	drawdonut(&ic, x + 55, y, (float)40, (float)60);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)60, (float)40, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)40, (float)60, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"60/40 - 40/60", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)75, (float)25);
-	drawdonut(&ic, x + 55, y, (float)25, (float)75);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)75, (float)25, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)25, (float)75, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"75/25 - 25/75", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)90, (float)10);
-	drawdonut(&ic, x + 55, y, (float)10, (float)90);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)90, (float)10, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)10, (float)90, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"90/10 - 10/90", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)95, (float)5);
-	drawdonut(&ic, x + 55, y, (float)5, (float)95);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)95, (float)5, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)5, (float)95, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"95/5 - 5/95", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)99, (float)1);
-	drawdonut(&ic, x + 55, y, (float)1, (float)99);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)99, (float)1, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)1, (float)99, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"99/1 - 1/99", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)99.9, (float)0.1);
-	drawdonut(&ic, x + 55, y, (float)0.1, (float)99.9);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)99.9, (float)0.1, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)0.1, (float)99.9, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"99.9/0.1 - 0.1/99.9", ic.ctext);
 
 	/* line 2 */
 	x = 40;
 	y = 160;
-	drawdonut(&ic, x, y, (float)0, (float)0);
-	drawdonut(&ic, x + 55, y, (float)25, (float)25);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)0, (float)0, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)25, (float)25, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"0/0 - 25/25", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)50, (float)0);
-	drawdonut(&ic, x + 55, y, (float)0, (float)50);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)50, (float)0, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)0, (float)50, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"50/0 - 0/50", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)40, (float)30);
-	drawdonut(&ic, x + 55, y, (float)30, (float)40);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)40, (float)30, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)30, (float)40, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"40/30 - 30/40", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)30, (float)20);
-	drawdonut(&ic, x + 55, y, (float)20, (float)30);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)30, (float)20, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)20, (float)30, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"30/20 - 20/30", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)20, (float)10);
-	drawdonut(&ic, x + 55, y, (float)10, (float)20);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)20, (float)10, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)10, (float)20, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"20/10 - 10/20", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)15, (float)5);
-	drawdonut(&ic, x + 55, y, (float)5, (float)15);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)15, (float)5, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)5, (float)15, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"15/5 - 5/15", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)10, (float)1);
-	drawdonut(&ic, x + 55, y, (float)1, (float)10);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)10, (float)1, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)1, (float)10, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"10/1 - 1/10", ic.ctext);
 
 	x += 130;
-	drawdonut(&ic, x, y, (float)1, (float)0.1);
-	drawdonut(&ic, x + 55, y, (float)0.1, (float)1);
+	drawdonut_libgd_bug_workaround(&ic, x, y, (float)1, (float)0.1, 49, 15);
+	drawdonut_libgd_bug_workaround(&ic, x + 55, y, (float)0.1, (float)1, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"1.0/0.1 - 0.1/1.0", ic.ctext);
 
 	/* line 3 */
 	x = 40;
 	y = 270;
 	gdImageStringUp(ic.im, gdFontGetSmall(), 1, y + 105, (unsigned char *)"libgd native", ic.ctext);
-	drawdonut_libgd_native(&ic, x, y, (float)0, (float)0);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)50, (float)50);
+	drawdonut_libgd_native(&ic, x, y, (float)0, (float)0, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)50, (float)50, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"0/0 - 50/50", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)100, (float)0);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)0, (float)100);
+	drawdonut_libgd_native(&ic, x, y, (float)100, (float)0, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)0, (float)100, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"100/0 - 0/100", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)60, (float)40);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)40, (float)60);
+	drawdonut_libgd_native(&ic, x, y, (float)60, (float)40, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)40, (float)60, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"60/40 - 40/60", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)75, (float)25);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)25, (float)75);
+	drawdonut_libgd_native(&ic, x, y, (float)75, (float)25, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)25, (float)75, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"75/25 - 25/75", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)90, (float)10);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)10, (float)90);
+	drawdonut_libgd_native(&ic, x, y, (float)90, (float)10, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)10, (float)90, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"90/10 - 10/90", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)95, (float)5);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)5, (float)95);
+	drawdonut_libgd_native(&ic, x, y, (float)95, (float)5, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)5, (float)95, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"95/5 - 5/95", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)99, (float)1);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)1, (float)99);
+	drawdonut_libgd_native(&ic, x, y, (float)99, (float)1, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)1, (float)99, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"99/1 - 1/99", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)99.9, (float)0.1);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)0.1, (float)99.9);
+	drawdonut_libgd_native(&ic, x, y, (float)99.9, (float)0.1, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)0.1, (float)99.9, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"99.9/0.1 - 0.1/99.9", ic.ctext);
 
 	/* line 4 */
 	x = 40;
 	y = 350;
-	drawdonut_libgd_native(&ic, x, y, (float)0, (float)0);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)25, (float)25);
+	drawdonut_libgd_native(&ic, x, y, (float)0, (float)0, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)25, (float)25, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"0/0 - 25/25", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)50, (float)0);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)0, (float)50);
+	drawdonut_libgd_native(&ic, x, y, (float)50, (float)0, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)0, (float)50, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"50/0 - 0/50", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)40, (float)30);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)30, (float)40);
+	drawdonut_libgd_native(&ic, x, y, (float)40, (float)30, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)30, (float)40, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"40/30 - 30/40", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)30, (float)20);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)20, (float)30);
+	drawdonut_libgd_native(&ic, x, y, (float)30, (float)20, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)20, (float)30, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"30/20 - 20/30", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)20, (float)10);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)10, (float)20);
+	drawdonut_libgd_native(&ic, x, y, (float)20, (float)10, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)10, (float)20, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"20/10 - 10/20", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)15, (float)5);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)5, (float)15);
+	drawdonut_libgd_native(&ic, x, y, (float)15, (float)5, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)5, (float)15, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"15/5 - 5/15", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)10, (float)1);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)1, (float)10);
+	drawdonut_libgd_native(&ic, x, y, (float)10, (float)1, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)1, (float)10, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"10/1 - 1/10", ic.ctext);
 
 	x += 130;
-	drawdonut_libgd_native(&ic, x, y, (float)1, (float)0.1);
-	drawdonut_libgd_native(&ic, x + 55, y, (float)0.1, (float)1);
+	drawdonut_libgd_native(&ic, x, y, (float)1, (float)0.1, 49, 15);
+	drawdonut_libgd_native(&ic, x + 55, y, (float)0.1, (float)1, 49, 15);
 	gdImageString(ic.im, gdFontGetSmall(), x - 20, y + 30, (unsigned char *)"1.0/0.1 - 0.1/1.0", ic.ctext);
 
 	gdImagePng(ic.im, pngout);
@@ -603,8 +605,8 @@ END_TEST
 
 START_TEST(element_output_check)
 {
-	int ret, x, y;
-	float i;
+	int ret, x, y, i;
+	float f;
 	char buffer[6];
 	IMAGECONTENT ic;
 	FILE *pngout;
@@ -612,10 +614,8 @@ START_TEST(element_output_check)
 	x = 1500;
 	y = 900;
 
-	defaultcfg();
 	initimagecontent(&ic);
-	ic.im = gdImageCreate(x, y);
-	colorinit(&ic);
+	imageinit(&ic, x, y);
 	ic.interface.updated = (time_t)get_timestamp(2012, 3, 4, 5, 6);
 	layoutinit(&ic, "donut with 0.2% input steps and other elements", x, y);
 
@@ -627,16 +627,16 @@ START_TEST(element_output_check)
 
 	gdImageStringUp(ic.im, gdFontGetSmall(), 1, y + 15, (unsigned char *)"50.0%", ic.ctext);
 
-	for (i = 50.0; i >= 0; i -= (float)0.2) {
+	for (f = 50.0; f >= 0; f -= (float)0.2) {
 
-		drawdonut(&ic, x, y, i, i);
+		drawdonut(&ic, x, y, f, f, 49, 15);
 		x += 55;
 
 		if (x > 1000) {
 			x = 40;
 			y += 60;
 
-			snprintf(buffer, 6, "%3.1f%%", (double)i - 0.2);
+			snprintf(buffer, 6, "%3.1f%%", (double)f - 0.2);
 			gdImageStringUp(ic.im, gdFontGetSmall(), 1, y + 15, (unsigned char *)buffer, ic.ctext);
 		}
 	}
@@ -647,31 +647,61 @@ START_TEST(element_output_check)
 	gdImageString(ic.im, gdFontGetSmall(), 1020, 100, (unsigned char *)"Small - The quick brown fox jumps over the lazy dog", ic.ctext);
 	gdImageString(ic.im, gdFontGetTiny(), 1020, 120, (unsigned char *)"Tiny - The quick brown fox jumps over the lazy dog", ic.ctext);
 
-	drawlegend(&ic, 1230, 140);
+	drawlegend(&ic, 1130, 140, 0);
+	drawlegend(&ic, 1330, 140, 1);
 
-	drawbar(&ic, 1050, 160, 400, 50, 50, 100);
-	drawbar(&ic, 1050, 180, 400, 25, 75, 100);
-	drawbar(&ic, 1050, 200, 400, 75, 25, 100);
-	drawbar(&ic, 1050, 220, 400, 0, 100, 100);
-	drawbar(&ic, 1050, 240, 400, 100, 0, 100);
+	drawbar(&ic, 1050, 160, 100, 50, 50, 100, 0);
+	drawbar(&ic, 1050, 180, 100, 25, 75, 100, 0);
+	drawbar(&ic, 1050, 200, 100, 75, 25, 100, 0);
+	drawbar(&ic, 1050, 220, 100, 0, 100, 100, 0);
+	drawbar(&ic, 1050, 240, 100, 100, 0, 100, 0);
 
-	drawbar(&ic, 1050, 260, 400, 50, 50, 130);
-	drawbar(&ic, 1050, 280, 400, 25, 75, 130);
-	drawbar(&ic, 1050, 300, 400, 75, 25, 130);
-	drawbar(&ic, 1050, 320, 400, 0, 100, 130);
-	drawbar(&ic, 1050, 340, 400, 100, 0, 130);
+	drawbar(&ic, 1050, 260, 100, 1, 99, 100, 0);
+	drawbar(&ic, 1050, 280, 100, 2, 98, 100, 0);
 
-	drawpole(&ic, 1050, 360, 400, 50, 50, 100);
-	drawpole(&ic, 1070, 360, 400, 25, 75, 100);
-	drawpole(&ic, 1090, 360, 400, 75, 25, 100);
-	drawpole(&ic, 1110, 360, 400, 0, 100, 100);
-	drawpole(&ic, 1130, 360, 400, 100, 0, 100);
+	drawbar(&ic, 1050, 300, 100, 99, 1, 100, 0);
+	drawbar(&ic, 1050, 320, 100, 98, 2, 100, 0);
 
-	drawpole(&ic, 1150, 360, 400, 50, 50, 130);
-	drawpole(&ic, 1170, 360, 400, 25, 75, 130);
-	drawpole(&ic, 1190, 360, 400, 75, 25, 130);
-	drawpole(&ic, 1210, 360, 400, 0, 100, 130);
-	drawpole(&ic, 1230, 360, 400, 100, 0, 130);
+	drawbar(&ic, 1200, 160, 100, 1, 0, 100, 0);
+	drawbar(&ic, 1200, 180, 100, 2, 0, 100, 0);
+	drawbar(&ic, 1200, 200, 100, 3, 0, 100, 0);
+
+	drawbar(&ic, 1200, 220, 100, 0, 1, 100, 0);
+	drawbar(&ic, 1200, 240, 100, 0, 2, 100, 0);
+	drawbar(&ic, 1200, 260, 100, 0, 3, 100, 0);
+
+	drawbar(&ic, 1200, 280, 100, 25, 25, 100, 0);
+	drawbar(&ic, 1200, 300, 100, 10, 30, 100, 0);
+	drawbar(&ic, 1200, 320, 100, 30, 10, 100, 0);
+
+	gdImageLine(ic.im, 1040, 360, 1260, 360, ic.ctext);
+	gdImageLine(ic.im, 1040, 760, 1260, 760, ic.ctext);
+
+	gdImageLine(ic.im, 1250, 350, 1250, 770, ic.ctext);
+
+	drawarrowup(&ic, 1250, 350);
+	drawarrowright(&ic, 1260, 360);
+
+	drawpoles(&ic, 1050, 360, 400, 50, 50, 100);
+	drawpoles(&ic, 1070, 360, 400, 25, 75, 100);
+	drawpoles(&ic, 1090, 360, 400, 75, 25, 100);
+	drawpoles(&ic, 1110, 360, 400, 0, 100, 100);
+	drawpoles(&ic, 1130, 360, 400, 100, 0, 100);
+
+	drawpoles(&ic, 1150, 360, 400, 50, 50, 130);
+	drawpoles(&ic, 1170, 360, 400, 25, 75, 130);
+	drawpoles(&ic, 1190, 360, 400, 75, 25, 130);
+	drawpoles(&ic, 1210, 360, 400, 0, 100, 130);
+	drawpoles(&ic, 1230, 360, 400, 100, 0, 130);
+
+	gdImageLine(ic.im, 1040, 870, 1160, 870, ic.ctext);
+	gdImageLine(ic.im, 1040, 820, 1160, 820, ic.ctext);
+	gdImageLine(ic.im, 1040, 770, 1160, 770, ic.ctext);
+
+	for (i = 0; i < 100; i++) {
+		drawpole(&ic, 1050 + i, 819, i % 50, 1, ic.crx);
+		drawpole(&ic, 1050 + i, 821, i % 50, 2, ic.ctx);
+	}
 
 	gdImageString(ic.im, gdFontGetMediumBold(), 1280, 400, (unsigned char *)"Color: ctext", ic.ctext);
 	gdImageString(ic.im, gdFontGetMediumBold(), 1280, 420, (unsigned char *)"Color: cedge", ic.cedge);
@@ -695,6 +725,152 @@ START_TEST(element_output_check)
 }
 END_TEST
 
+START_TEST(hextorgb_can_convert)
+{
+	debug = 1;
+	int rgb[3];
+
+	suppress_output();
+
+	rgb[0] = 1;
+	rgb[1] = 2;
+	rgb[2] = 3;
+	hextorgb("000000", rgb);
+	ck_assert_int_eq(rgb[0], 0);
+	ck_assert_int_eq(rgb[1], 0);
+	ck_assert_int_eq(rgb[2], 0);
+
+	rgb[0] = 1;
+	rgb[1] = 2;
+	rgb[2] = 3;
+	hextorgb("#000000", rgb);
+	ck_assert_int_eq(rgb[0], 0);
+	ck_assert_int_eq(rgb[1], 0);
+	ck_assert_int_eq(rgb[2], 0);
+
+	rgb[0] = 1;
+	rgb[1] = 2;
+	rgb[2] = 3;
+	hextorgb("FFFFFF", rgb);
+	ck_assert_int_eq(rgb[0], 255);
+	ck_assert_int_eq(rgb[1], 255);
+	ck_assert_int_eq(rgb[2], 255);
+
+	rgb[0] = 1;
+	rgb[1] = 2;
+	rgb[2] = 3;
+	hextorgb("#FFFFFF", rgb);
+	ck_assert_int_eq(rgb[0], 255);
+	ck_assert_int_eq(rgb[1], 255);
+	ck_assert_int_eq(rgb[2], 255);
+
+	rgb[0] = 1;
+	rgb[1] = 2;
+	rgb[2] = 3;
+	hextorgb("ABCABBA", rgb);
+	ck_assert_int_eq(rgb[0], 171);
+	ck_assert_int_eq(rgb[1], 202);
+	ck_assert_int_eq(rgb[2], 187);
+
+	rgb[0] = 1;
+	rgb[1] = 2;
+	rgb[2] = 3;
+	hextorgb("#ABCABBA", rgb);
+	ck_assert_int_eq(rgb[0], 171);
+	ck_assert_int_eq(rgb[1], 202);
+	ck_assert_int_eq(rgb[2], 187);
+}
+END_TEST
+
+START_TEST(modcolor_mods_colors)
+{
+	debug = 1;
+	int rgb[3];
+
+	suppress_output();
+
+	rgb[0] = 10;
+	rgb[1] = 20;
+	rgb[2] = 30;
+	modcolor(rgb, 10, 0);
+	ck_assert_int_eq(rgb[0], 20);
+	ck_assert_int_eq(rgb[1], 30);
+	ck_assert_int_eq(rgb[2], 40);
+
+	rgb[0] = 10;
+	rgb[1] = 20;
+	rgb[2] = 30;
+	modcolor(rgb, -10, 0);
+	ck_assert_int_eq(rgb[0], 0);
+	ck_assert_int_eq(rgb[1], 10);
+	ck_assert_int_eq(rgb[2], 20);
+
+	rgb[0] = 10;
+	rgb[1] = 20;
+	rgb[2] = 30;
+	modcolor(rgb, -20, 0);
+	ck_assert_int_eq(rgb[0], 0);
+	ck_assert_int_eq(rgb[1], 0);
+	ck_assert_int_eq(rgb[2], 10);
+
+	rgb[0] = 10;
+	rgb[1] = 20;
+	rgb[2] = 30;
+	modcolor(rgb, -30, 0);
+	ck_assert_int_eq(rgb[0], 40);
+	ck_assert_int_eq(rgb[1], 50);
+	ck_assert_int_eq(rgb[2], 60);
+
+	rgb[0] = 10;
+	rgb[1] = 20;
+	rgb[2] = 30;
+	modcolor(rgb, -30, 1);
+	ck_assert_int_eq(rgb[0], 0);
+	ck_assert_int_eq(rgb[1], 0);
+	ck_assert_int_eq(rgb[2], 0);
+
+	rgb[0] = 10;
+	rgb[1] = 250;
+	rgb[2] = 30;
+	modcolor(rgb, 30, 0);
+	ck_assert_int_eq(rgb[0], 40);
+	ck_assert_int_eq(rgb[1], 255);
+	ck_assert_int_eq(rgb[2], 60);
+
+	rgb[0] = 10;
+	rgb[1] = 250;
+	rgb[2] = 30;
+	modcolor(rgb, 30, 1);
+	ck_assert_int_eq(rgb[0], 40);
+	ck_assert_int_eq(rgb[1], 255);
+	ck_assert_int_eq(rgb[2], 60);
+
+	rgb[0] = 10;
+	rgb[1] = 250;
+	rgb[2] = 251;
+	modcolor(rgb, 30, 0);
+	ck_assert_int_eq(rgb[0], 0);
+	ck_assert_int_eq(rgb[1], 220);
+	ck_assert_int_eq(rgb[2], 221);
+
+	rgb[0] = 10;
+	rgb[1] = 250;
+	rgb[2] = 251;
+	modcolor(rgb, 30, 1);
+	ck_assert_int_eq(rgb[0], 40);
+	ck_assert_int_eq(rgb[1], 255);
+	ck_assert_int_eq(rgb[2], 255);
+
+	rgb[0] = 10;
+	rgb[1] = 20;
+	rgb[2] = 251;
+	modcolor(rgb, -30, 0);
+	ck_assert_int_eq(rgb[0], 40);
+	ck_assert_int_eq(rgb[1], 50);
+	ck_assert_int_eq(rgb[2], 255);
+}
+END_TEST
+
 void add_image_tests(Suite *s)
 {
 	TCase *tc_image = tcase_create("Image");
@@ -702,7 +878,7 @@ void add_image_tests(Suite *s)
 	tcase_add_unchecked_fixture(tc_image, setup, teardown);
 	tcase_set_timeout(tc_image, 10);
 	tcase_add_test(tc_image, initimagecontent_does_not_crash);
-	tcase_add_test(tc_image, colorinit_does_not_crash);
+	tcase_add_test(tc_image, imageinit_does_not_crash);
 	tcase_add_test(tc_image, layoutinit_does_not_crash);
 	tcase_add_test(tc_image, getimagevalue_zeropadding);
 	tcase_add_test(tc_image, getimagevalue_normal);
@@ -719,5 +895,7 @@ void add_image_tests(Suite *s)
 	tcase_add_test(tc_image, hourly_imagescaling_rate_1000);
 	tcase_add_test(tc_image, libgd_output_comparison);
 	tcase_add_test(tc_image, element_output_check);
+	tcase_add_test(tc_image, hextorgb_can_convert);
+	tcase_add_test(tc_image, modcolor_mods_colors);
 	suite_add_tcase(s, tc_image);
 }
