@@ -76,7 +76,6 @@ START_TEST(vnstat_parseargs_can_modify_settings)
 					"--end", "2001-01-01", NULL};
 	int argc = sizeof(argv) / sizeof(char *) - 1;
 
-	defaultcfg();
 	initparams(&p);
 	suppress_output();
 	parseargs(&p, argc, argv);
@@ -512,7 +511,6 @@ START_TEST(vnstat_parseargs_limit_changes_defaults)
 	char *argv[] = {"vnstat", "--limit", "1234", NULL};
 	int argc = sizeof(argv) / sizeof(char *) - 1;
 
-	defaultcfg();
 	initparams(&p);
 	suppress_output();
 	parseargs(&p, argc, argv);
@@ -533,7 +531,6 @@ START_TEST(vnstat_parseargs_limit_overrides)
 	char *argv[] = {"vnstat", "-d", "5", "--limit", "234", NULL};
 	int argc = sizeof(argv) / sizeof(char *) - 1;
 
-	defaultcfg();
 	initparams(&p);
 	suppress_output();
 	parseargs(&p, argc, argv);
@@ -554,7 +551,6 @@ START_TEST(vnstat_parseargs_limit_overrides_regardless_of_position)
 	char *argv[] = {"vnstat", "--limit", "345", "-d", "5", NULL};
 	int argc = sizeof(argv) / sizeof(char *) - 1;
 
-	defaultcfg();
 	initparams(&p);
 	suppress_output();
 	parseargs(&p, argc, argv);
@@ -572,7 +568,7 @@ END_TEST
 START_TEST(vnstat_parseargs_iflist_gives_help)
 {
 	PARAMS p;
-	char *argv[] = {"vnstat", "--iflist", "notknown", NULL};
+	char *argv[] = {"vnstat", "--iflist", "--help", NULL};
 	int argc = sizeof(argv) / sizeof(char *) - 1;
 
 	initparams(&p);
@@ -753,6 +749,70 @@ START_TEST(vnstat_parseargs_dbiflist_with_parseable_mode)
 }
 END_TEST
 
+START_TEST(vnstat_parseargs_can_select_interface_without_parameter)
+{
+	PARAMS p;
+	char *argv[] = {"vnstat", "ethsomething", NULL};
+	int argc = sizeof(argv) / sizeof(char *) - 1;
+
+	defaultcfg();
+	initparams(&p);
+	suppress_output();
+	parseargs(&p, argc, argv);
+	ck_assert_str_eq(p.interface, "ethsomething");
+	ck_assert_int_eq(p.defaultiface, 0);
+}
+END_TEST
+
+START_TEST(vnstat_parseargs_can_select_interface_without_parameter_even_if_there_are_other_parameters)
+{
+	PARAMS p;
+	char *argv[] = {"vnstat", "-d", "ethsomethingtoo", NULL};
+	int argc = sizeof(argv) / sizeof(char *) - 1;
+
+	defaultcfg();
+	initparams(&p);
+	suppress_output();
+	ck_assert_int_ne(cfg.qmode, 1);
+	parseargs(&p, argc, argv);
+	ck_assert_str_eq(p.interface, "ethsomethingtoo");
+	ck_assert_int_eq(p.defaultiface, 0);
+	ck_assert_int_eq(cfg.qmode, 1);
+}
+END_TEST
+
+START_TEST(vnstat_parseargs_can_select_interface_without_parameter_from_the_middle_of_the_parameter_list)
+{
+	PARAMS p;
+	char *argv[] = {"vnstat", "-m", "ethsomethings", "-ru", NULL};
+	int argc = sizeof(argv) / sizeof(char *) - 1;
+
+	defaultcfg();
+	initparams(&p);
+	suppress_output();
+	debug = 1;
+	ck_assert_int_ne(cfg.qmode, 2);
+	parseargs(&p, argc, argv);
+	ck_assert_str_eq(p.interface, "ethsomethings");
+	ck_assert_int_eq(p.defaultiface, 0);
+	ck_assert_int_eq(cfg.qmode, 2);
+}
+END_TEST
+
+START_TEST(vnstat_parseargs_knows_when_interface_name_is_too_long_even_without_parameter)
+{
+	PARAMS p;
+	char *argv[] = {"vnstat", "ethethethethethethethethethetheth", NULL};
+	int argc = sizeof(argv) / sizeof(char *) - 1;
+
+	defaultcfg();
+	initparams(&p);
+	suppress_output();
+	parseargs(&p, argc, argv);
+	ck_assert_int_eq(p.defaultiface, 1);
+}
+END_TEST
+
 void add_parseargs_tests(Suite *s)
 {
 	TCase *tc_pa = tcase_create("ParseArgs");
@@ -809,5 +869,9 @@ void add_parseargs_tests(Suite *s)
 	tcase_add_exit_test(tc_pa, vnstat_parseargs_dbiflist_without_parameters, 0);
 	tcase_add_exit_test(tc_pa, vnstat_parseargs_dbiflist_with_verbose_mode, 0);
 	tcase_add_exit_test(tc_pa, vnstat_parseargs_dbiflist_with_parseable_mode, 0);
+	tcase_add_test(tc_pa, vnstat_parseargs_can_select_interface_without_parameter);
+	tcase_add_test(tc_pa, vnstat_parseargs_can_select_interface_without_parameter_even_if_there_are_other_parameters);
+	tcase_add_test(tc_pa, vnstat_parseargs_can_select_interface_without_parameter_from_the_middle_of_the_parameter_list);
+	tcase_add_exit_test(tc_pa, vnstat_parseargs_knows_when_interface_name_is_too_long_even_without_parameter, 1);
 	suite_add_tcase(s, tc_pa);
 }
