@@ -1,4 +1,5 @@
 #include "common.h"
+#include "cfg.h"
 #include "vnstat_tests.h"
 #include "common_tests.h"
 #include "dbsql_tests.h"
@@ -79,11 +80,12 @@ void verify_fork_status(void)
 }
 
 void setup(void) {
+	defaultcfg();
 	debug = 0;
 }
 
 void teardown(void) {
-	;
+	return;
 }
 
 void suppress_output(void)
@@ -353,5 +355,9 @@ uint64_t get_timestamp(const int year, const int month, const int day, const int
 	stm.tm_min = minute;
 	stm.tm_isdst = -1;
 
-	return (uint64_t)mktime(&stm);
+	if (cfg.useutc) {
+		return (uint64_t)timegm(&stm);
+	} else {
+		return (uint64_t)mktime(&stm);
+	}
 }
